/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import org.ow2.petals.launcher.configuration.Configuration;
import org.ow2.petals.launcher.exception.PetalsLauncherException;

/**
 * A class loader that load jars *recursively* from the the lib directory and from the extension directory
 * 
 * @author Sebastien Andre - EBM WebSourcing
 */
public class PetalsLauncherClassLoader extends URLClassLoader {

    final static String PETALS_KERNEL_JAR_REGEXP = "petals-microkernel-(\\d+)(\\.(\\d+))?(\\.(\\d+))?(-.*)?\\.jar";

    private final static FilenameFilter KERNEL_FILTER = new FilenameFilter() {
        @Override
        public boolean accept(final File dir, final String name) {
            return name.matches(PETALS_KERNEL_JAR_REGEXP);
        }
    };

    private final static FilenameFilter JAR_FILTER = new FilenameFilter() {
        @Override
        public boolean accept(final File dir, final String name) {
            return name.endsWith(".jar");
        }
    };

    private final static FileFilter DIR_FILTER = new FileFilter() {
        @Override
        public boolean accept(final File pathname) {
            return pathname.isDirectory();
        }
    };


    public PetalsLauncherClassLoader(final Configuration config) throws PetalsLauncherException {
        super(new URL[] {}, PetalsLauncherClassLoader.class.getClassLoader());

        try {

            // Add the kenel JAR to the classloader
            // -> it will also load the dependencies from its manifest
            // and make them visible in this current classloader
            for (File jar : config.getLibDirectory().listFiles(KERNEL_FILTER)) {
                addFile(jar);
            }

            // Add the microkernel component implementations to the classloader
            // This is needed to load the registry implementation for example
            // TODO reference the dependencies from the manifest of the implementation?
            // NOTE: we must use recursion because there could be many subdirectories
            this.addRecursivelyJars(config.getImplementationsDirectory());

            // Add the microkernel extensions to the classloader
            // this is needed to load all the extensions
            // TODO have one directory per extension?
            // TODO reference the dependencies from the manifest of the extension?
            final File extDir = config.getExtensionsDirectory();
            if (extDir != null) {
                if (extDir.exists()) {
                    if (extDir.isDirectory()) {
                        addDirectoryJars(extDir);
                    } else {
                        throw new PetalsLauncherException("The provided extension directory '"
                                + extDir + "' is not a directory.");
                    }
                }
            }
        } catch (PetalsLauncherException e) {
            throw e;
        } catch (Exception e) {
            throw new PetalsLauncherException(e);
        }

    }

    private void addRecursivelyJars(final File dir) throws MalformedURLException {
        for (final File file : dir.listFiles(JAR_FILTER)) {
            addFile(file);
        }
        for (final File file : dir.listFiles(DIR_FILTER)) {
            this.addRecursivelyJars(file);
        }
    }

    private void addDirectoryJars(final File dir) throws MalformedURLException {
        for (File jar : dir.listFiles(JAR_FILTER)) {
            addFile(jar);
        }
    }

    private void addFile(final File f) throws MalformedURLException {
        super.addURL(f.toURI().toURL());
    }
}
